/*
 * feedbackd - dynamic feedback system for LVS
 * Copyright (C) 2002 Jeremy Kerr
 * 
 * This file is part of feedbackd.
 *
 *  feedbackd is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  feedbackd is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with feedbackd; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**
 * @file constant.c
 *
 * Sample load reporting plugin.
 */

#include <libxml/tree.h>
#include <string.h>
#include <errno.h>

#include "plugin.h"

typedef struct {
	unsigned int health;
} constant_config;

void (*lh)(loglevel, const char *, ...) = NULL;

void _init()
{
	lh = NULL;
}

void loghandle(void (*logh)(loglevel, const char *, ...))
{
	lh = logh;
}

void *config(xmlNodePtr node)
{
	char *value, *end;
	constant_config *cfg = NULL;
	unsigned long tmp;
	
	node = node->children;
	while (node) {
		if (xmlIsBlankNode(node)) {
			node = node->next;
			continue;
		}
		if (!xmlStrcmp(node->name, (const xmlChar*)"value")) {
			if (!(value = xmlNodeListGetString(node->doc,
					             node->children, 1))) {
				if (lh)
					(*lh)(LOG_ERR, "[PLUGIN constant] "
						        "no value specified");
				return NULL;
			}
			
			cfg = (constant_config *)
				malloc(sizeof(constant_config));
			if (!cfg) {
				if (lh)
					(*lh)(LOG_ERR, "[PLUGIN constant] "
					               "malloc: %s",
						       strerror(errno));
				return NULL;
			}
			value = xmlNodeListGetString(node->doc,
			                             node->children, 1);
			errno = 0;
			tmp = strtoul(value, &end, 10);
			if (*end != '\0' || end == value || errno == ERANGE
			    || tmp > 100) {
				if (lh)
					(*lh)(LOG_ERR, "Invalid string (%s) "
					               "given for constant "
						       "health value", value);
				free(cfg);
				return NULL;
			}

			cfg->health = (unsigned int)tmp;

			if (lh)
				(*lh)(LOG_INFO, "[PLUGIN constant] Set "
					         "constant health value to "
						 "%d", cfg->health);
			
			break;
		}
		node = node->next;
	}
	if (!cfg->health && lh)
		(*lh)(LOG_ERR, "[PLUGIN constant] No <value> tag in config");

	return cfg;
}	

int get_health(void *cfg)
{
	return ((constant_config *)cfg)->health;
}

int destroy(void *cfg) {
	free(cfg);
	return 0;
}

void _fini()
{
}
