/*
 * feedbackd - dynamic feedback system for LVS
 * Copyright (C) 2002 Jeremy Kerr
 * 
 * This file is part of feedbackd.
 *
 *  feedbackd is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  feedbackd is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with feedbackd; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**
 * @file plugin.h
 *
 * Header file for feedbackd-agent plugins
 */


/**
 * Called when the module is first loaded.
 */
void _init(void);


/**
 * Log level options.
 */
typedef enum {
	LOG_CRIT	= 1, /**< Really important stuff */
	LOG_ERR		= 2, /**< Errors */
	LOG_WARN	= 3, /**< Warnings */
	LOG_INFO	= 4, /**< Information */
	LOG_DEBUG	= 5, /**< Debugging */
	LOG_VDEBUG	= 6  /**< Debugging, with packet data */
} loglevel;

/**
 * Pass a log handle to the plugin. This function is optional: if present,
 * feedbackd-agent will call it with a reference to the log() function, to
 * allow the plugin to write to the currently used log stream. This plugin
 * requires the feedback-agent log.h header to be present.
 *
 * @param logh A reference to the log() function
 */
void loghandle(void (*logh)(loglevel, const char *, ...));


/* only include this if libxml is present */
#ifdef LIBXML_VERSION
/**
 * Configures the plugin. This function is optional: if present,
 * feedbackd-agent will pass the <monitor> XML node to the plugin to allow
 * arbitrary configuration values to be passed. This function requires the
 * libxml2 headers to be present. It's probably a good idea to use the log
 * handle in this function, as users will want some sort of reason why
 * configuration failed. The loghandle function will be called before this.
 *
 * If this function is implemented, the destroy() function should also be
 * provided to perform clean up operations, and free the pointer to config
 * data.
 *
 * @param node The <monitor> node containing the configuration for this plugin
 * @return A pointer to the plugin's config data if the configuration was
 *         successful, zero otherwise.
 */
void *config(xmlNodePtr node);
#endif

/**
 * Get the current system load, given a configuration structure.
 * 
 * @param configdata the configuration data to use 
 * @return the load - a value between 0 and 100
 */
int get_health(void *configdata);


#ifdef LIBXML_VERSION
/**
 * Perform any clean-up operations on a monitor configuration. This function
 * is only required if configuration data exists; so depends on the config()
 * function being present.
 *
 * @param configdata the configuration data used by this plugin
 * @return zero on sucess, non zero otherwise
 */
int destroy(void *configdata);
#endif

/**
 * Called when the module is unloaded
 */
void _fini(void);
